#!/usr/bin/env python3
"""
🍳 CloudCostChefs: OCI Compartment Tagger (Generic Edition)

Reads a local CSV and applies common tags to OCI compartments.
Designed for readability—focuses on 4–5 widely used tags: Environment, Owner, CostCenter, Project, and optionally Application.
"""

import os
import logging
import pandas as pd
import oci

# =============================
# Configuration & Logging Setup
# =============================

# Path to OCI config file (update if needed)
OCI_CONFIG_PATH = os.getenv('OCI_CONFIG_PATH', os.path.expanduser('~/.oci/config'))
OCI_PROFILE = os.getenv('OCI_PROFILE', 'DEFAULT')

# Local CSV file path (update to your CSV location)
CSV_PATH = os.getenv('COMPARTMENT_CSV_PATH', './Compartment_Tags.csv')

# Log directory & file
LOG_DIR = os.getenv('LOG_DIR', './logs')
os.makedirs(LOG_DIR, exist_ok=True)
LOG_FILENAME = os.path.join(LOG_DIR, 'oci_compartment_tagger.log')

logging.basicConfig(
    filename=LOG_FILENAME,
    level=logging.INFO,
    format='%(asctime)s [%(levelname)s] %(message)s'
)
console_handler = logging.StreamHandler()
console_handler.setLevel(logging.INFO)
console_formatter = logging.Formatter('%(asctime)s [%(levelname)s] %(message)s')
console_handler.setFormatter(console_formatter)
logging.getLogger().addHandler(console_handler)

# =============================
# OCI Client Initialization
# =============================

logging.info("Initializing OCI configuration and Identity client.")
config = oci.config.from_file(OCI_CONFIG_PATH, OCI_PROFILE)
identity_client = oci.identity.IdentityClient(config)


# =============================
# Helper Functions
# =============================

def prepare_tags(row):
    """
    Build a dictionary of defined tags from a CSV row for a generic namespace.
    Uses the common tags: Environment, Owner, CostCenter, Project, Application (optional).
    """
    namespace = 'CloudCostChefs'
    defined_tags = {namespace: {}}

    if pd.notnull(row.get('Environment')):
        defined_tags[namespace]['Environment'] = str(row['Environment']).strip()

    if pd.notnull(row.get('Owner')):
        defined_tags[namespace]['Owner'] = str(row['Owner']).strip()

    if pd.notnull(row.get('CostCenter')):
        defined_tags[namespace]['CostCenter'] = str(row['CostCenter']).strip()

    if pd.notnull(row.get('Project')):
        defined_tags[namespace]['Project'] = str(row['Project']).strip()

    # Optional additional tag
    if 'Application' in row and pd.notnull(row.get('Application')):
        defined_tags[namespace]['Application'] = str(row['Application']).strip()

    # If no common tags found, return empty dict
    if not defined_tags[namespace]:
        return {}

    return defined_tags


def get_compartment_ocid(name, parent_id):
    """
    Recursively search for a compartment by name under a given parent compartment ID.
    Returns a tuple: (found_ocid, parent_of_found_ocid).
    Raises ValueError if not found.
    """
    compartments = identity_client.list_compartments(
        parent_id,
        compartment_id_in_subtree=True,
        access_level="ACCESSIBLE"
    ).data

    for comp in compartments:
        if comp.name == name:
            return comp.id, comp.compartment_id

    # If not found directly, search recursively
    for comp in compartments:
        try:
            return get_compartment_ocid(name, comp.id)
        except ValueError:
            continue

    raise ValueError(f"Compartment '{name}' not found.")


def update_compartment_tags(compartment_id, new_tags):
    """
    Merge new defined tags with any existing defined tags on the compartment,
    then update the compartment with merged tags.
    """
    compartment = identity_client.get_compartment(compartment_id).data
    merged_tags = compartment.defined_tags or {}
    for ns, tags_dict in new_tags.items():
        if ns not in merged_tags:
            merged_tags[ns] = {}
        merged_tags[ns].update(tags_dict)

    update_details = oci.identity.models.UpdateCompartmentDetails(
        defined_tags=merged_tags
    )
    identity_client.update_compartment(compartment_id, update_details)
    logging.info(f"Updated tags for compartment OCID: {compartment_id}")


# =============================
# Main Tagging Logic
# =============================

def main():
    logging.info("Starting generic OCI compartment tagging process.")

    # Load CSV into DataFrame
    try:
        df = pd.read_csv(CSV_PATH, encoding='utf-8')
    except Exception as e:
        logging.error(f"Failed to load CSV at '{CSV_PATH}': {e}")
        return

    # Validate required columns (at least one of the common tags plus CompartmentName)
    required_columns = ['CompartmentName']
    if not all(col in df.columns for col in required_columns):
        logging.error(f"Missing required CSV column: 'CompartmentName'")
        return

    # Iterate through each row and apply tags
    for idx, row in df.iterrows():
        compartment_name = str(row.get('CompartmentName', '')).strip()
        if not compartment_name:
            logging.warning(f"Row {idx} has empty CompartmentName. Skipping.")
            continue

        tags = prepare_tags(row)
        if not tags:
            logging.info(f"No common tags found for '{compartment_name}'. Skipping.")
            continue

        try:
            ocid, parent_id = get_compartment_ocid(compartment_name, config['tenancy'])
            update_compartment_tags(ocid, tags)
            logging.info(f"Successfully tagged '{compartment_name}' (OCID: {ocid})")
        except ValueError as ve:
            logging.warning(f"{ve} Skipping.")
        except Exception as ex:
            logging.error(f"Error tagging '{compartment_name}': {ex}")

    logging.info("Generic OCI compartment tagging process complete.")


if __name__ == "__main__":
    main()